public inbox for gcc-cvs@sourceware.org
help / color / mirror / Atom feed
* [gcc r12-5598] Fix -freorder-blocks-and-partition glitch with Windows SEH (continued)
@ 2021-11-30  9:19 Eric Botcazou
  0 siblings, 0 replies; only message in thread
From: Eric Botcazou @ 2021-11-30  9:19 UTC (permalink / raw)
  To: gcc-cvs

https://gcc.gnu.org/g:bc8d6c60137f8bbf173b86ddf31b15d7ba2a33dd

commit r12-5598-gbc8d6c60137f8bbf173b86ddf31b15d7ba2a33dd
Author: Eric Botcazou <ebotcazou@adacore.com>
Date:   Tue Nov 30 10:17:09 2021 +0100

    Fix -freorder-blocks-and-partition glitch with Windows SEH (continued)
    
    This fixes a thinko in the fix for the -freorder-blocks-and-partition
    glitch with SEH on 64-bit Windows:
      https://gcc.gnu.org/pipermail/gcc-patches/2021-February/565208.html
    
    Even if no exceptions are active, e.g. in C, we need to consider calls.
    
    gcc/
            PR target/103274
            * config/i386/i386.c (ix86_output_call_insn): Beef up comment about
            nops emitted with SEH.
            * config/i386/winnt.c (i386_pe_seh_unwind_emit): When switching to
            the cold section, emit a nop before the directive if the previous
            active instruction is a call.

Diff:
---
 gcc/config/i386/i386.c  | 6 ++++--
 gcc/config/i386/winnt.c | 4 ++--
 2 files changed, 6 insertions(+), 4 deletions(-)

diff --git a/gcc/config/i386/i386.c b/gcc/config/i386/i386.c
index 2657e7817ae..0e6bf3e0fef 100644
--- a/gcc/config/i386/i386.c
+++ b/gcc/config/i386/i386.c
@@ -16438,8 +16438,10 @@ ix86_output_call_insn (rtx_insn *insn, rtx call_op)
 	    break;
 
 	  /* If we get to the epilogue note, prevent a catch region from
-	     being adjacent to the standard epilogue sequence.  If non-
-	     call-exceptions, we'll have done this during epilogue emission. */
+	     being adjacent to the standard epilogue sequence.  Note that,
+	     if non-call exceptions are enabled, we already did it during
+	     epilogue expansion, or else, if the insn can throw internally,
+	     we already did it during the reorg pass.  */
 	  if (NOTE_P (i) && NOTE_KIND (i) == NOTE_INSN_EPILOGUE_BEG
 	      && !flag_non_call_exceptions
 	      && !can_throw_internal (insn))
diff --git a/gcc/config/i386/winnt.c b/gcc/config/i386/winnt.c
index 7c0ea4f731c..0aaf46f050a 100644
--- a/gcc/config/i386/winnt.c
+++ b/gcc/config/i386/winnt.c
@@ -1243,9 +1243,9 @@ i386_pe_seh_unwind_emit (FILE *out_file, rtx_insn *insn)
   seh = cfun->machine->seh;
   if (NOTE_P (insn) && NOTE_KIND (insn) == NOTE_INSN_SWITCH_TEXT_SECTIONS)
     {
       /* See ix86_seh_fixup_eh_fallthru for the rationale.  */
       rtx_insn *prev = prev_active_insn (insn);
-      if (prev && !insn_nothrow_p (prev))
+      if (prev && (CALL_P (prev) || !insn_nothrow_p (prev)))
 	fputs ("\tnop\n", out_file);
       fputs ("\t.seh_endproc\n", out_file);
       seh->in_cold_section = true;


^ permalink raw reply	[flat|nested] only message in thread

only message in thread, other threads:[~2021-11-30  9:19 UTC | newest]

Thread overview: (only message) (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
@@ -1243,9 +1243,9 @@ i386_pe_seh_unwind_emit (FILE *out_file, rtx_insn *insn)
   if (NOTE_P (insn) && NOTE_KIND (insn) == NOTE_INSN_SWITCH_TEXT_SECTIONS)
     {
-      /* See ix86_seh_fixup_eh_fallthru for the rationale.  */
+      /* See ix86_output_call_insn/seh_fixup_eh_fallthru for the rationale.  */
       rtx_insn *prev = prev_active_insn (insn);
2021-11-30  9:19 [gcc r12-5598] Fix -freorder-blocks-and-partition glitch with Windows SEH (continued) Eric Botcazou

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).